--[[
#########################################################################################
# THE BOARD CLASS - handles the board upon which the game takes place					#
#########################################################################################
# Belongs to the LÖVE-ly Tetris project													#
# tv_user - 2012																		#
# Free to use for all purposes															#
#########################################################################################
--]]

cBoard = {}
cBoard.__index = cBoard


-- THIS FUNCTION CREATES THE BOARD CLASS
function cBoard.create(bx, by) -- bx and by are the coordinates to the top-left corner of the board
	
	local temp = {} -- first create a temp table
	setmetatable(temp, cBoard) -- make its metatable this very own (cBoard)
	-- set initial vars
	temp.Board = {}
	temp.PosX = bx
	temp.PosY = by
	temp.nLinDel = 0
	return temp
	
end


-- Just empties the main board array
function cBoard:ResetBoard()
	for i = 1, bH do
		self.Board[i] = {}
		for j = 1, bW do
			self.Board[i][j] = 0
		end
	end
	self.nLinDel = 0
end


-- Stores a given piece in a given position (pX,pY)
function cBoard:StorePiece (pX, pY, piece, rotation)
	-- cycle through each block of the piece array, and if it isn't empty, store it in the board array
	for i = 1, nPBlocks do
		for j = 1, nPBlocks do
			if (cPieces.PieceStruct[piece][rotation][i][j] ~= 0) then
				if (((pY+i-1) > 0) and ((pX+j-1) > 0)) then -- just a precaution...
					self.Board[pY+i-1][pX+j-1] = piece
				end	
			end
		end
	end
end


-- Checks if it is game over, i.e., if there is a piece in the top line
function cBoard:IsGameOver ()
	--cycle the entire top line to see if any space is occupied. if so, return true, else, return false
	for i = 1, bW do
		if (self.Board[1][i] ~= 0) then return true end
	end
	-- loop ended and no space was occupied, so return false
	return false
end


-- Deletes a given line
function cBoard:DeleteLine (line)
	-- works towards the top, copying the line above to the one below, starting at the given line
	for i = line, 2, -1 do
		for j = 1, bW do
			self.Board[i][j] = self.Board[i-1][j]
		end
	end
	self.nLinDel = self.nLinDel + 1
end


-- checks if a line is full and returns a boolean result
function cBoard:IsLineFull (line)
	local temp = 0
	for i = 1, bW do
		if (self.Board[line][i] ~= 0) then temp = temp + 1 end
	end
	return (temp == bW)
end


-- checks if a board is empty (to generate a victory event!)
function cBoard:IsEmpty ()
	local temp = true
	-- do a reverse check (from bottom to top), because the place where we will most 
	-- likely find stored pieces is the bottom. This way we save some loop time
	for i = bH, 1, -1 do
		for j = 0, bW do
			if (self.Board[i][j] ~= 0) then
				temp = false
				break
			end	
		end	
	end
	return temp
end


-- checks if a line is full, and if so, deletes it (calls the method .DeleteLine)
function cBoard:DeletePossibleLines ()
	local temp = 0
	for i = 1, bH do
		temp = 0
		for j = 1, bW do
			if (self.Board[i][j] ~= 0) then temp = temp + 1 end
		end
		if (temp == bW) then self:DeleteLine (i) end
	end
	-- checks if map is empty, and returns boolean
	return self:IsEmpty()
end


-- Checks if a given position for a piece is possible, or if there is a collision (with boundaries or stored blocks)
function cBoard:IsPossibleMovement (pX, pY, piece, rotation)
	for i = 1, nPBlocks do
		for j = 1, nPBlocks do
			if (cPieces.PieceStruct[piece][rotation][i][j] ~= 0) then
				if (((pX+j-1) < 1) or ((pX+j-1) > bW) or ((pY+i-1) > bH)) then
					return false
				elseif ((pY+i-1) > 0) then
					if (self.Board[(pY+i-1)][(pX+j-1)] ~= 0) then return false end
				end	
			end
		end
	end
	-- no collision
	return true
end


-- Draws the two board limits and all the board blocks
function cBoard:DrawBoard()
	-- coordenadas do campo sup esq: (self.PosX ; self.PosY)

 	--1. Draw the stored blocks
	for i = 1, bH do
		for j = 1, bW do
			-- check if block is filled and if so, draw a rectangle
			if (self.Board[i][j] ~= 0) then
				love.graphics.setColor(color[self.Board[i][j]]) -- use color of the given block
				love.graphics.rectangle("fill", self.PosX + ((j-1) * blockSize), self.PosY + ((i-1) * blockSize), blockSize, blockSize)
			end
		end
	end
	-- 2. Draw a grid
	-- first set line options
	love.graphics.setLine(1, "rough")
	love.graphics.setColor(color["dark"])
	-- Horizontal lines
	for i = 1, bH + 1 do
		love.graphics.line(self.PosX, self.PosY + ((i-1) * blockSize), self.PosX + (bW * blockSize), self.PosY + ((i-1) * blockSize))
	end	
	-- Vertical lines
	for i = 1, bW + 1 do
		love.graphics.line(self.PosX + ((i-1) * blockSize), self.PosY , self.PosX + ((i-1) * blockSize), self.PosY + (bH * blockSize))
	end
    -- 3. Draw the boundaries
    love.graphics.setColor(color["foreground"])
	love.graphics.line(self.PosX, self.PosY, self.PosX, self.PosY + (bH * blockSize))
	love.graphics.line(self.PosX + (bW * blockSize), self.PosY, self.PosX + (bW * blockSize), self.PosY + (bH * blockSize))
	love.graphics.line(self.PosX, self.PosY + (bH * blockSize), self.PosX + (bW * blockSize), self.PosY + (bH * blockSize))
	
end